/****************************************************************************************//**
* \file				i2cDriver.c
*
* \brief This unit is a simple driver library for I2C bus based on the Open-Core I2C-master.
* 
* \author PYD : 12/6/2013
* \version 0.1
* \date 12/6/2013

* PYD : 12/6/2013 initial version
* PYD : 01/10/2018 remove code for ES12 and ES3 versions
*//******************************************************************************************/
//================================================================================
//                 USER ORIENTED PROCEDURES FOR EXTERNAL USE
#include "i2cDriver.h"

//========================= SYSTEM CONFIGURATION =================================
#include "pcie40_definitions.h"
#include "systemConfig.h"
#include "pcie40_ecs.h"

#include <stdio.h>
#include <error.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define offsetReg(r) (r*4)



// addresses of I2C registers (start at 0xa40 in qsys)
#define I2C_PRELOREG  0x0
#define I2C_PREHIREG  0x1
#define I2C_CTRLREG   0x2
#define I2C_TSMTREG   0x3
#define I2C_RCVREG    0x3
#define I2C_CMDREG    0x4
#define I2C_STATREG   0x4

// Control register
#define I2C_ENA      0x80 
#define I2C_DIS      0x00 
// Command register bits 
#define I2C_STA      0x80 
#define I2C_STO      0x40 
#define I2C_RD       0x20 
#define I2C_WR       0x10 
#define I2C_NACK     0x08 
#define I2C_IACK     0x01 
// Status register bits
#define I2C_TIP      0x02 
#define I2C_BUSY     0x40

unsigned lastSetInCLK[MAX_DEV];
unsigned LastSetSCLfreq[MAX_DEV];

static int getBusBase(int bus){
	switch(bus){
		case I2C_BUS_TEMPERATURES:
			return(I2C_BUS_BASE_TEMPERATURES);
		break;
		case I2C_BUS_PLL_CLEANER1:
			return(I2C_BUS_BASE_PLL_CLEANER1);
		break;
		case I2C_BUS_PLL_CLEANER2:
			return(I2C_BUS_BASE_PLL_CLEANER2);
		break;
		case I2C_BUS_PLL_TFC:
			return(I2C_BUS_BASE_PLL_TFC);
		break;
		case I2C_BUS_SFP1:
			return(I2C_BUS_BASE_SFP1);
		break;
		case I2C_BUS_SFP2:
			return(I2C_BUS_BASE_SFP2);
		break;
		case I2C_BUS_MINIPODS:
			return(I2C_BUS_BASE_MINIPODS);
		break;
		case I2C_BUS_TEMP_MEZZANINE:
			return(I2C_BUS_BASE_TEMP_MEZZANINE);
		break;
		case I2C_BUS_CURRENT1:
			return(I2C_BUS_BASE_CURRENT1);
		break;
		case I2C_BUS_CURRENT2:
			return(I2C_BUS_BASE_CURRENT2);
		break;
		case I2C_BUS_FANOUT:
		    return(I2C_BUS_BASE_FANOUT);
                break;
		case I2C_BUS_FPGA_EEPROM:
		    return(I2C_BUS_BASE_FPGA_EEPROM);
		default:
			printf("ERROR: unknown i2c bus number %d in file %s line %d\n",bus,__FILE__,__LINE__);
			return(99);
	}
}
 /***********************************************************************//**
 * \fn void i2c_enable(int dev, int bus)
 * \brief Function to activate the i2c-opencore IP.
 * \param dev: the board number
 * \param bus: the target i2c bus number
 *//***********************************************************************/
static void i2c_enable(int dev, int bus) {
	unsigned cval = I2C_ENA;

	ecs_iowrI2c_slow( dev, getBusBase(bus)+  (I2C_CTRLREG*4), &cval);
}
/***********************************************************************//**
 * \fn void i2c_disable(int dev,int bus)
 * \param dev: the board number
 * \brief Function to deactivate the i2c-opencore IP.
 * \param bus: the target i2c bus number
 *//***********************************************************************/
static void i2c_disable(int dev,int bus) {
	unsigned cval = I2C_DIS;

	ecs_iowrI2c_slow( dev, getBusBase(bus)+  (I2C_CTRLREG*4), &cval);		
}
/***********************************************************************//**
 * \fn void i2c_stop(int dev,int bus)
 * \param dev: the board number
 * \brief Function to generate the stop condition on the i2c bus.
 * \param bus: the target i2c bus number
 *//***********************************************************************/
static void i2c_stop(int dev, int bus) {
	unsigned cval = I2C_STO;

	ecs_iowrI2c_slow( dev, getBusBase(bus)+  (I2C_CMDREG*4), &cval);	
}

/****************************************************************************//**
 * \fn void i2c_setSpeed(int dev, int bus, int value)
 * \brief Function to program the prescale the SCL clock line. Should be called
 *        before the I2C use.
 * \param dev: the board number
 * \param   bus: the target i2c bus number
 * \param value: the value to write in the prescale register
 *//**************************************************************************/
static void i2c_setSpeed(int dev, int bus, int value ) {
	unsigned cval = value>>8;
	int 	 base = getBusBase(bus);

	ecs_iowrI2c_slow(dev, base + (I2C_PREHIREG*4), &cval);
	cval =value & 0xFF;
	ecs_iowrI2c_slow(dev, base + (I2C_PRELOREG*4), &cval);
}
/****************************************************************************//**
 * \fn int i2c_init(int dev, int bus, unsigned inCLK, unsigned SCLfreq)
 * \brief Function to open access to i2c opencore and set speed
 * \      if it is specified
 * \param dev: the board number
 * \param     bus: the target i2c bus number
 * \param   inCLK: the input system clock ferquency in Hz
 * \param SCLfreq: the target SCL clock frequency in Hz
 * \return 0 success,-1 error
 *//**************************************************************************/
int i2c_init(int dev, int bus, unsigned inCLK, unsigned SCLfreq) {
	long int prescale;

//	printf("Simulation init with clock %d fequency %d \n",inCLK, SCLfreq); return(0);

	if (ecs_openLli(dev)<0){
		printf("ERROR: Can't init I2C\n");
		return(-1);
	}
	i2c_stop(dev, bus);
	i2c_disable(dev, bus);
	i2c_enable(dev, bus);
	// set prescale register
	if (inCLK!=0 && SCLfreq !=0){
		prescale = (inCLK/(5*SCLfreq))-1;
		i2c_setSpeed(dev, bus, prescale);
        	lastSetInCLK[dev] = inCLK;
        	LastSetSCLfreq[dev] = SCLfreq;
	}
	return(0);
}

/****************************************************************************//**
 * \fn int i2c_getInit(int dev, int bus, unsigned &inCLK, unsigned &SCLfreq)
 * \brief Read the last values in the prescale of the SCL clock line. 
 *
 * \param dev: the board number
 * \param     bus: the target i2c bus number
 * \param   inCLK: the system clock ferquency in Hz
 * \param SCLfreq: the target SCL clock frequency in Hz
 *//**************************************************************************/
void i2c_getInit(int dev, int bus, unsigned *inCLK, unsigned *SCLfreq) {
	*inCLK = lastSetInCLK[dev];
	*SCLfreq = LastSetSCLfreq[dev];
}
/***********************************************************************************//**
 * \fn int _waitTIP()
 * \brief function to wait for the status register TIP bit value 0 "transfer complete"
 *
 * \param     dev: the borad number
 * \param     bus: the target i2c bus number       
 * \return 0 success,-1 error
 *//********************************************************************************/
static int i2c_waitTIP(int dev, int bus) {
	unsigned val, step = 0;
	int 	 base = getBusBase(bus);
	while(1){
      usleep(10); //MJ POUR TEMPO_FERNAND LE 12 SEPT 2017
		step = step + 1;
		if(ecs_iordI2c_slow(dev, base + (I2C_STATREG*4), &val)==0){
			if ((val & I2C_TIP) == 0){ //transfer complete
				return(0);
			} else { // transfer in progress
				if (step > 500){
					printf("ERROR: timeout in i2c_waitTIP()\n");
					return(-1);
				} else {
					usleep(10);
				}
			}
		} else { // read error
			printf("i2c_waitTIP firmware level read error KO\n");
			return(-1);
		}
	}//while
}

/***********************************************************************//**
 * \fn int i2c_readMem(int dev, int bus, unsigned slaveAdd, unsigned regIndex, unsigned *val)
 * \brief Function to read one single register
 *     
 * \param     dev: the board number
 * \param      bus: the target i2c bus number          
 * \param slaveAdd: the address of the device to write in the I2C space
 * \param regIndex: the number of the register in the component
 * \param val     : a pointer to receive the read value
 * \return 0 success,-1 error
 *//***********************************************************************/
int i2c_readMem(int dev, int bus, unsigned slaveAdd, unsigned regIndex, unsigned *val) {
	unsigned cval;
	int 	 base = getBusBase(bus);

	cval = slaveAdd<<1; 
	ecs_iowrI2c_slow(dev, base +  (I2C_TSMTREG*4), &cval);
	// set STA bit and WR bit in commandReg
	cval = I2C_STA | I2C_WR;
	ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);
	// wait fir TIP flaf in statusReg
	if ( i2c_waitTIP(dev, bus) != 0) 
			return(-1);
	// read statusReg to verify RxACK=0
	ecs_iordI2c_slow(dev, base +  (I2C_STATREG*4), &cval);
	if ((cval & I2C_NACK) == I2C_NACK){
		printf("ERROR: bus=%d i2c_readMem() first transaction slave address not acknowledge in I2C_Write\n",bus);
		return(-1);
	}
	//
	// write $regIndex in transmitReg
	//
	cval = regIndex;
	ecs_iowrI2c_slow(dev, base +  (I2C_TSMTREG*4), &cval);
	// set WR bit in commandReg
	cval = I2C_WR;
	ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);
	// wait fir TIP flaf in statusReg
	if ( i2c_waitTIP(dev, bus) != 0)
			return(-1);
	// read statusReg to verify RxACK=0
	ecs_iordI2c_slow(dev, base +  (I2C_STATREG*4), &cval);
	if ((cval & I2C_NACK) == I2C_NACK){
		printf("ERROR:bus %d i2c_readMem() second transaction slave address not acknowledge in I2C_Write\n",bus);
		return(-1);
	}
	//
	// write $slaveAdd (slaveAdd+RD bit) in transmitReg
	//
	cval = slaveAdd<<1 | 1;
	ecs_iowrI2c_slow(dev, base +  (I2C_TSMTREG*4), &cval);	
	// set STA bit and WR bit in commandReg
	cval = I2C_STA | I2C_WR;
	ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);
	// wait fir TIP flaf in statusReg
	if ( i2c_waitTIP(dev, bus) != 0)
			return(-1);
	// read statusReg to verify RxACK=0
	if ((cval & I2C_NACK) == I2C_NACK){
		printf("ERROR: bus %d i2c_readMem() third transaction slave address not acknowledge in I2C_Write\n",bus);
		return(-1);
	}
	//
	// set RD bit, set ACK bit =1(NACK) set STO bit in commandReg
	//
	cval = I2C_RD | I2C_NACK | I2C_STO;
	ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);
	// wait for TIP flag in statusReg
	if ( i2c_waitTIP(dev, bus) != 0)
			return(-1);	
	return( ecs_iordI2c_slow(dev, base +  (I2C_RCVREG*4), val) );
}
/***********************************************************************//**
 * \fn int i2c_writeMem(int dev, int bus, unsigned slaveAdd, unsigned regIndex, unsigned *val)
 * \brief Function to write a value in a register
 *     
 * \param     dev: the board number
 * \param      bus: the target i2c bus number      
 * \param slaveAdd: the address of the device to write in the I2C space
 * \param regIndex: the number of the register in the component
 * \param val     : a pointer to the value to write
 * \return 0 success,-1 error
 *//***********************************************************************/
int i2c_writeMem(int dev, int bus, unsigned slaveAdd, unsigned regIndex, unsigned *val){
	unsigned cval;
	int 	 base = getBusBase(bus);

	// generate start command 
	// write $slaveAdd (slaveAdd+WR bit) in transmitReg
	cval = slaveAdd<<1; 
	ecs_iowrI2c_slow(dev, base +  (I2C_TSMTREG*4), &cval);
	// set STA bit and WR bit in commandReg
	cval = I2C_STA | I2C_WR;
	ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);
	// wait fir TIP flaf in statusReg
	if ( i2c_waitTIP(dev, bus) != 0) 
			return(-1);
	// read statusReg to verify RxACK=0
	ecs_iordI2c_slow(dev, base +  (I2C_STATREG*4), &cval);
	if ((cval & I2C_NACK) == I2C_NACK){
		printf("ERROR: bus %d slave address not acknowledge in I2C_Write\n",bus);
		return(-1);
	}
	//
	// write $regIndex in transmitReg
	//
	cval = regIndex;
	ecs_iowrI2c_slow(dev, base +  (I2C_TSMTREG*4), &cval);
	// set WR bit in commandReg
	cval = I2C_WR;
	ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);
	// wait fir TIP flaf in statusReg
	if ( i2c_waitTIP(dev, bus) != 0) 
			return(-1);
	// read statusReg to verify RxACK=0
	ecs_iordI2c_slow(dev, base +  (I2C_STATREG*4), &cval);
	if ((cval & I2C_NACK) == I2C_NACK){
		printf("ERROR: bus %d slave address not acknowledge in I2C_Write\n",bus);
		return(-1);
	}
	//
	// write $slaveAdd (slaveAdd+RD bit) in transmitReg
	//
	ecs_iowrI2c_slow(dev, base +  (I2C_TSMTREG*4), val);	
	// set STA bit and WR bit in commandReg
	cval = I2C_STO | I2C_WR;
	ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);	
	// wait fir TIP flaf in statusReg
	if ( i2c_waitTIP(dev, bus) != 0) 
			return(-1);
return(0);
}
/***********************************************************************//**
 * \fn int i2c_multReadMemInit(int dev, int bus, unsigned slaveAdd, unsigned regIndex)
 * \brief Function to initialize a multiread sequence from slave at
 *        i2c address slaveAdd starting at device internal register regIndex
 *       
 * \param     dev: the board number
 * \param      bus: the target i2c bus number    
 * \param slaveAdd: the address of the device to read from in the I2C space
 * \param regIndex: the number of the first register in the sequence to read
 * \return 0 success,-1 error
 *//***********************************************************************/
static int i2c_multReadMemInit(int dev, int bus, unsigned slaveAdd, unsigned regIndex)  {
	unsigned cval;
	int 	 base = getBusBase(bus);

	// generate start command 
	// write $slaveAdd (slaveAdd+WR bit) in transmitReg
	//
	cval = slaveAdd<<1; 
	ecs_iowrI2c_slow(dev, base +  (I2C_TSMTREG*4), &cval);
	// set STA bit and WR bit in commandReg
	cval = I2C_STA | I2C_WR;
	ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);
	// wait for TIP flag in statusReg
	if ( i2c_waitTIP(dev, bus) != 0) 
			return(-1);
	// read statusReg to verify RxACK=0
	ecs_iordI2c_slow(dev, base +  (I2C_STATREG*4), &cval);
	if ((cval & I2C_NACK) == I2C_NACK){
		printf("ERROR: bus %d slave address not acknowledge in I2C_Write\n",bus);
		return(-1);
	}
	//
	// write $regIndex in transmitReg
	//
	cval = regIndex;
	ecs_iowrI2c_slow(dev, base +  (I2C_TSMTREG*4), &cval);
	// set WR bit in commandReg
	cval = I2C_WR;
	ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);
	// wait fir TIP flaf in statusReg
	if ( i2c_waitTIP(dev, bus) != 0)
			return(-1);
	// read statusReg to verify RxACK=0
	ecs_iordI2c_slow(dev, base +  (I2C_STATREG*4), &cval);
	if ((cval & I2C_NACK) == I2C_NACK){
		printf("ERROR: bus %d slave address not acknowledge in I2C_Write\n",bus);
		return(-1);
	}
	//
	// write $slaveAdd (slaveAdd+RD bit) in transmitReg
	//
	cval = slaveAdd<<1 | 1;
	ecs_iowrI2c_slow(dev, base +  (I2C_TSMTREG*4), &cval);	
	// set STA bit and WR bit in commandReg
	cval = I2C_STA | I2C_WR;
	ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);
	// wait fir TIP flaf in statusReg
	if ( i2c_waitTIP(dev, bus) != 0)
			return(-1);
	return(0);
}
/******************************************************************************//**
 * \fn int i2c_multReadMem(int dev, int bus, unsigned slaveAdd, unsigned regIndex, int nb)
 * \brief Function to read/print a sequence of nb conscecutive registers from slave at
 *        i2c address slaveAdd starting at device internal register regIndex
 *        Used with nb=1 is a function to read one register value
 *
 * This function allocate the array to store the read data in. The user
 * should free it after use.
 *       
 * \param     dev: the board number
 * \param      bus: the target i2c bus number    
 * \param slaveAdd: the address of the device to read from in the I2C space
 * \param regIndex: the number of the first register in the sequence to read
 * \param nb      : the number of conscecutives registers to read
 * \param data    : a pointer to receive the data array reference
 * \return 0 success,-1 error
 *//******************************************************************************/
int i2c_multReadMem(int dev, int bus, unsigned slaveAdd, unsigned regIndex, int nb, unsigned **data) {
	unsigned cval;
	int i;
	unsigned *bloc;

	int 	 base = getBusBase(bus);

	if (i2c_multReadMemInit(dev, bus, slaveAdd, regIndex) != 0)
			return(-1);
        bloc = (unsigned *)malloc(nb*sizeof(unsigned));
	if (bloc==0){
		perror("Can't malloc in i2c_multReadMem()\n");
		return(-1);
	}
	// Read and display values
	cval = I2C_RD;

	for (i=0; i<(nb-1); i++){
		ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);
		// wait for TIP flag in statusReg
		if ( i2c_waitTIP(dev, bus) != 0)
			return(-1);
		ecs_iordI2c_slow(dev, base +  (I2C_RCVREG*4), &bloc[i]);
	}

	// last byte to transfer
	cval = I2C_RD | I2C_NACK | I2C_STO;
	ecs_iowrI2c_slow(dev, base +  (I2C_CMDREG*4), &cval);
	// wait for TIP flag in statusReg
		if ( i2c_waitTIP(dev, bus) != 0)
			return(-1);
	ecs_iordI2c_slow(dev, base +  (I2C_RCVREG*4), &bloc[nb-1]);
	*data = bloc;
	return(0);
}
